#include "convolution.h"


// ,   a[k] = a[k] * b[k] * scale, 0 <= k < n.
__global__ void mult_scale(cufftComplex * a, const cufftComplex * b, int n, float scale)
{
    const int num_threads = blockDim.x * gridDim.x;
    const int thread_id = blockIdx.x * blockDim.x + threadIdx.x;
    for (int k = thread_id; k < n; k += num_threads)
    {
        float ax = (a[k].x * b[k].x - a[k].y * b[k].y) * scale;
        float ay = (a[k].x * b[k].y + a[k].y * b[k].x) * scale;
        a[k].x = ax;
        a[k].y = ay;
    }
}


//       ,
//   a  b   .
void convolve_gpu(cufftComplex * a, cufftComplex * b, int n, cufftComplex * result)
{
    //  plan  .
    cufftHandle plan;
    cufftPlan1d(&plan, n, CUFFT_C2C, 1);

    //    a  b    .
    cufftExecC2C(plan, a, a, CUFFT_FORWARD);
    cufftExecC2C(plan, b, b, CUFFT_FORWARD);

    //      ,
    //    a (  ..   
    //   CUFFT   ).
    mult_scale<<<4, 256>>>(a, b, n, 1.0f / n);

    //    a,    result.
    cufftExecC2C(plan, a, result, CUFFT_INVERSE);

    //  plan.
    cufftDestroy(plan);
}
